/**
 * @license
 * Copyright 2022 Dynatrace LLC
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import Window from './polyfills/window';
import { createContext } from './context';

it('should be instanceof Window', () => {
  const context = createContext(process.cwd(), '/api/test', {
    appId: 'my.app.id',
    appName: 'my-app-name',
    environmentUrl: 'http://localhost.test',
    token: 'my-token,',
  }).vmContext;

  expect(context).toBeInstanceOf(Window);
  expect(context.self).toBeInstanceOf(Window);

  const propertyDescriptor = Object.getOwnPropertyDescriptor(
    context,
    'Window',
  )!;
  expect(propertyDescriptor.configurable).toBe(true);
  expect(propertyDescriptor.enumerable).toBe(false);
  expect(propertyDescriptor.writable).toBe(true);
});

it('should have global variables', () => {
  const config = {
    appId: 'my.app.id',
    appName: 'my.app.name',
    environmentUrl: 'http://localhost.test',
    tenantId: 'my-tenant',
  };

  const context = createContext(process.cwd(), '/api/test', {
    ...config,
    token: 'my-token',
  }).vmContext;

  for (const [key, value] of Object.entries(config)) {
    expect(context[key]).toBe(value);

    const propertyDescriptor = Object.getOwnPropertyDescriptor(context, key)!;
    expect(propertyDescriptor.configurable).toBe(true);
    expect(propertyDescriptor.enumerable).toBe(true);
    expect(propertyDescriptor.writable).toBe(false);
  }
});

it('atob, btoa, location, AbortSignal', () => {
  const context = createContext(process.cwd(), '/api/test', {
    appId: 'my.app.id',
    appName: 'my.app.name',
    environmentUrl: 'http://localhost.test',
    token: 'my-token,',
  }).vmContext;

  expect(context.btoa('test')).toBe('dGVzdA==');
  expect(context.atob('dGVzdA==')).toBe('test');
  expect(context.location.href).toBe('http://localhost.test/api/test');
  expect(context.AbortSignal.abort()).toBeInstanceOf(AbortSignal);
});
