/**
 * @license
 * Copyright 2022 Dynatrace LLC
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

enum DOM_EXCEPTIONS {
  IndexSizeError = 1,
  DOMStringSizeError = 2,
  HierarchyRequestError = 3,
  WrongDocumentError = 4,
  InvalidCharacterError = 5,
  NoDataAllowedError = 6,
  NoModificationAllowedError = 7,
  NotFoundError = 8,
  NotSupportedError = 9,
  InUseAttributeError = 10,
  InvalidStateError = 11,
  SyntaxError = 12,
  InvalidModificationError = 13,
  NamespaceError = 14,
  InvalidAccessError = 15,
  ValidationError = 16,
  TypeMismatchError = 17,
  SecurityError = 18,
  NetworkError = 19,
  AbortError = 20,
  URLMismatchError = 21,
  QuotaExceededError = 22,
  TimeoutError = 23,
  InvalidNodeTypeError = 24,
  DataCloneError = 25,
}

export default class DOMException extends Error {
  /** */
  constructor(
    message?: string,
    readonly name: keyof typeof DOM_EXCEPTIONS | string = 'Error',
  ) {
    super(message);
  }

  /** Returns a short that contains one of the error code constants, or 0 if none match */
  get code() {
    return DOM_EXCEPTIONS[this.name];
  }
}
