/**
 * @license
 * Copyright 2022 Dynatrace LLC
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

interface ErrorEventInit extends EventInit {
  colno?: number;
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  error?: any;
  filename?: string;
  lineno?: number;
  message?: string;
}

export class ErrorEvent extends Event {
  readonly colno: number;
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  readonly error: any;
  readonly filename: string;
  readonly lineno: number;
  readonly message: string;
  /** */
  constructor(type: 'error', eventInitDict: ErrorEventInit = {}) {
    const {
      colno = 0,
      error = null,
      filename = '',
      lineno = 0,
      message = '',
      ...eventInit
    } = eventInitDict;
    super(type, eventInit);
    this.colno = colno;
    this.error = error;
    this.filename = filename;
    this.lineno = lineno;
    this.message = message;
  }
}

interface PromiseRejectionEventInit extends EventInit {
  promise?: Promise<unknown>;
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  reason?: any;
}

export class PromiseRejectionEvent extends Event {
  readonly promise?: Promise<unknown>;
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  readonly reason?: any;

  /** PromiseRejectionEvent constructor */
  constructor(
    type: 'unhandledrejection',
    eventInitDict: PromiseRejectionEventInit = {},
  ) {
    const { promise, reason, ...eventInit } = eventInitDict;
    super(type, eventInit);
    this.promise = promise;
    this.reason = reason;
  }
}

interface CustomEventInit extends EventInit {
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  readonly details?: any;
}

export class CustomEvent extends Event {
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  details: any;
  /** */
  constructor(type: string, options: CustomEventInit = {}) {
    const { details = null, ...eventInit } = options;
    super(type, eventInit);
    this.details = details;
  }
}

interface ProgressEventInit extends EventInit {
  lengthComputable?: boolean;
  loaded?: number;
  total?: number;
}

export class ProgressEvent<T extends EventTarget = EventTarget> extends Event {
  readonly lengthComputable: boolean;
  readonly loaded: number;
  readonly total: number;
  // eslint-disable-next-line @typescript-eslint/ban-ts-comment
  // @ts-ignore
  readonly target: T | null;
  /** */
  constructor(type: string, init: ProgressEventInit = {}) {
    const {
      lengthComputable = false,
      loaded = 0,
      total = 0,
      ...eventInit
    } = init;
    super(type, eventInit);
    this.lengthComputable = lengthComputable;
    this.loaded = loaded;
    this.total = total;
  }
}
