/**
 * @license
 * Copyright 2022 Dynatrace LLC
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

declare module 'vm' {
  interface ModuleEvaluateOptions {
    timeout?: number;
    breakOnSigint?: boolean;
  }

  type ModuleLinker = (
    specifier: string,
    referencingModule: Module,
  ) => Module | Promise<Module>;

  type ModuleStatus =
    | 'unlinked'
    | 'linking'
    | 'linked'
    | 'evaluating'
    | 'evaluated'
    | 'errored';

  class Module<Namespace = any> {
    context: Context;
    identifier: string;
    dependencySpecifiers: string[];
    namespace: Namespace;
    status: ModuleStatus;
    error: any;

    link(linker: ModuleLinker): Promise<Module>;

    evaluate(options?: ModuleEvaluateOptions): Promise<undefined>;
  }

  type SourceTextModuleInitializeImportMeta = (
    meta: any,
    module: SourceTextModule,
  ) => void;

  type SourceTextModuleImportModuleDynamically = (
    specifier: string,
    module: Module,
  ) => Promise<Module>;

  interface SourceTextModuleOptions {
    identifier?: string;
    cachedData?: Buffer | NodeJS.TypedArray | DataView;
    context?: Context;
    lineOffset?: number;
    columnOffset?: number;
    initializeImportMeta?: SourceTextModuleInitializeImportMeta;
    importModuleDynamically?: SourceTextModuleImportModuleDynamically;
  }

  class SourceTextModule<Namespace = any> extends Module<Namespace> {
    constructor(code: string, options?: SourceTextModuleOptions);

    createCachedData(): Buffer;
  }

  type SyntheticModuleEvaluate<Namespace> = (
    this: SyntheticModule<Namespace>,
  ) => void;

  interface SyntheticModuleOptions {
    identifier?: string;
    context?: Context;
  }

  class SyntheticModule<Namespace = any> extends Module<Namespace> {
    constructor(
      exportNames: (keyof Namespace)[],
      evaluateCallback: SyntheticModuleEvaluate<Namespace>,
      options?: SyntheticModuleOptions,
    );

    setExport(name: keyof Namespace, value: any): void;
  }
}
